# Copyright (C) 2009, 2010 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from unittest import TestCase

import xmlrpclib
from apache_openid.utils import HTTP_FORBIDDEN, OK, SERVER_RETURN
from apache_openid.utils.mock import ApacheMockRequest
from launchpad_lists_handler import (
    HTTP_SERVICE_UNAVAILABLE,
    LaunchpadListsHandler,
)
from mock import Mock, patch


class HandlerTestCase(TestCase):
    overrides = {
        'action-path': "/openid/",
    }

    def setUp(self):
        self.apache_request = ApacheMockRequest(overrides=self.overrides)
        self.handler = LaunchpadListsHandler(self.apache_request)
        self.patch = patch.object(self.handler, 'check_public_teams',
            self.mock_check_public_teams)

    def mock_check_public_teams(self, handler, authorized_teams):
        return LaunchpadListsHandler.check_public_teams(handler,
            authorized_teams)

    def assertRaisesMessage(self, exc, call, msg):
        try:
            result = call()
        except Exception, e:
            self.assertEqual(isinstance(e, exc), True)
            self.assertEqual(e.args[0], msg)
        else:
            self.fail("No exception raised. Got a result instead: %s" % result)


class CheckPublicTeamsErrorTestCase(HandlerTestCase):
    overrides = {
        'authorized-teams': 'some-team',
    }

    def test_team_denied_if_undetermined(self):
        # test the protect method by faking the check_public_teams
        # method response
        mock_check_public_teams = Mock(return_value=None)
        with patch.object(self.handler, 'check_public_teams',
                mock_check_public_teams):
            response = self.handler.protect()

        self.assertEqual(response, HTTP_SERVICE_UNAVAILABLE)

    def test_team_denied_on_rpc_error(self):
        self.handler.options['public-team-url'] = '/foo'
        self.handler.options['public-team-method'] = 'isTeamPublic'

        # fake an rpc exception
        with patch('xmlrpclib.ServerProxy') as mock_proxy:
            mock_proxy.return_value.isTeamPublic.side_effect = (
                xmlrpclib.ProtocolError(None, None, None, None))
            response = self.handler.protect()

        self.assertEqual(response, HTTP_SERVICE_UNAVAILABLE)


class PublicTeamsTestCase(HandlerTestCase):
    overrides = {
        'action-path': '/openid/',
        'authorized-teams': 'some-team',
    }

    def mock_check_public_teams(handler, authorized_teams):
        # apache_openid should give us ['some-team'] from the option overrides
        if 'some-team' in authorized_teams:
            return True
        return False

    def test_public_team_allowed(self):
        self.handler.request.request.subprocess_env['REQUEST_URI'] = '/protected-resource/'
        with self.patch:
            response = self.handler.protect()

        self.assertEqual(response, OK)


class PrivateTeamsTestCase(HandlerTestCase):
    overrides = {
        'authorized-teams': 'some-team',
    }

    def mock_check_public_teams(handler, authorized_teams):
        # always act like this is a non-public team
        return False

    def test_delegation_authorized(self):
        # requests for /favicon.ico are always allowed by apache_openid
        self.handler.request.request.subprocess_env['REQUEST_URI'] = '/favicon.ico'

        with self.patch:
            response = self.handler.protect()

        self.assertEqual(response, OK)

    def test_delegation_unauthorized(self):
        self.handler.request.request.subprocess_env['REQUEST_URI'] = '/protected-resource/'
        self.handler.request.request.method = 'POST'
        self.handler.request.request.status = None

        with self.patch:
            self.assertRaisesMessage(SERVER_RETURN, self.handler.protect,
                HTTP_FORBIDDEN)
